@@ -1,5 +1,9 @@
 import os
 import time
+from asyncio import AbstractEventLoop
+
+from consul.aio import Consul as AioConsul
+from aioch import Client as ClickhouseAioClient
 
 from singleton_decorator import singleton
 import hvac
@@ -16,6 +20,8 @@ import aiokafka
 from ssl import SSLContext
 import traceback
 import psycopg2
+import aioboto3
+import boto3
 
 
 def log(_func=None, *, logger = None):
@@ -84,9 +90,32 @@ logger = logging.getLogger(__name__)
 
 consul_keys = {"clickhouse": "env/databases/clickhouse", "kafka": "env/databases/kafka",
                "ms-azure-se": "env/databases/ms-azure-se", "loki": "env/databases/loki",
-               "greenplum": "env/databases/greenplum"}
-services = {"consul": {}, "kafka_producer": {}, "kafka_consumer": {}, "clickhouse_client": {}, "azure_container_client": {},
-            "azure_blob_client": {}, "loki_handler": {}, "aiokafka_producer": {}, "aiokafka_consumer": {}, "gp_connection": {}}
+               "greenplum": "env/databases/greenplum", "storage": "env/databases/storage",}
+services = {
+    "consul": {},
+    "aio_consul": {},
+
+    "kafka_producer": {},
+    "kafka_consumer": {},
+    "aiokafka_producer": {},
+    "aiokafka_consumer": {},
+
+    "clickhouse_client": {},
+    "aio_clickhouse": {},
+
+    "azure_container_client": {},
+    "azure_blob_client": {},
+
+    "loki_handler": {},
+
+    "gp_connection": {},
+
+    "aiostorage_client": {},
+    "aiostorage_resource": {},
+    "storage_client": {},
+    "storage_resource": {},
+}
+
 
 @singleton
 class Factory:
@@ -162,13 +191,22 @@ class Factory:
         return config
 
     def consul(self, **kwargs) -> consulate.Consul:
-        service_name = "consul"
         instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
-        scrts = self.secrets["consul"]
-        kwargs["host"] = scrts["url"] if not "url" in kwargs else kwargs["url"]
-        kwargs["port"] = scrts["port"] if not "port" in kwargs else kwargs["port"]
-        kwargs["token"] = scrts["backend_token"] if not "token" in kwargs else kwargs["token"]
-        return self.get_instance(consulate.Consul, service_name, instance_name, [], kwargs)
+        return self._consul_client(consulate.Consul, "consul", instance_name, kwargs)
+
+    def aio_consul(self, loop: AbstractEventLoop, **kwargs) -> AioConsul:
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        kwargs["loop"] = loop
+        return self._consul_client(AioConsul, "aio_consul", instance_name, kwargs)
+
+    def _consul_client(self, service_builder, service_name: str, instance_name: str, kwargs):
+        secrets = self.secrets["consul"]
+
+        kwargs["host"] = secrets["url"] if not "url" in kwargs else kwargs["url"]
+        kwargs["port"] = secrets["port"] if not "port" in kwargs else kwargs["port"]
+        kwargs["token"] = secrets["backend_token"] if not "token" in kwargs else kwargs["token"]
+
+        return self.get_instance(service_builder, service_name, instance_name, [], kwargs)
 
     def clickhouse_client(self, **kwargs) -> ch_d.Client:
         service_name = "clickhouse_client"
@@ -184,6 +222,24 @@ class Factory:
         kwargs["verify"] = False if "verify" not in kwargs else kwargs["verify"]
         return self.get_instance(ch_d.Client, service_name, instance_name, args, kwargs)
 
+    def aio_clickhouse(self, loop: AbstractEventLoop, **kwargs) -> ClickhouseAioClient:
+        service_name = "aio_clickhouse"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        secrets = self.secrets["clickhouse"]
+        config = self.config["clickhouse"]
+
+        args = []
+        args.append(config["url"] if "url" not in kwargs else kwargs["url"])
+        kwargs["port"] = config["port"] if not ("port" in kwargs) else kwargs["port"]
+        kwargs["user"] = secrets["user"] if not ("user" in kwargs) else kwargs["user"]
+        kwargs["password"] = secrets["password"] if not ("password" in kwargs) else kwargs["password"]
+        kwargs["secure"] = True if "secure" not in kwargs else kwargs["secure"]
+        kwargs["verify"] = False if "verify" not in kwargs else kwargs["verify"]
+
+        kwargs["loop"] = loop
+
+        return self.get_instance(ClickhouseAioClient, service_name, instance_name, args, kwargs)
+
     def kafka_producer(self, **kwargs) -> KafkaProducer:
         def serialiser(v):
             return json.dumps(v).encode('utf-8')
@@ -316,5 +372,48 @@ class Factory:
         kwargs["target_session_attrs"] = cnfg["target_session_attrs"] if not "target_session_attrs" in kwargs else kwargs["target_session_attrs"]
         return self.get_instance(psycopg2.connect, service_name, instance_name, [], kwargs)
 
+    def storage_client(self, **kwargs):
+        service_name = "storage_client"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        scrts = self.secrets["storage"]
+        cnfg = self.config["storage"]
+        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
+        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
+        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
+        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
+        return self.get_instance(boto3.Session().client, service_name, instance_name, [], kwargs)
 
 
+    def storage_resource(self, **kwargs):
+        service_name = "storage_resource"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        scrts = self.secrets["storage"]
+        cnfg = self.config["storage"]
+        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
+        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
+        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
+        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
+        return self.get_instance(boto3.resource, service_name, instance_name, [], kwargs)
+
+    def aiostorage_resource(self, **kwargs):
+        service_name = "aiostorage_resource"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        scrts = self.secrets["storage"]
+        cnfg = self.config["storage"]
+        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
+        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
+        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
+        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
+        return self.get_instance(aioboto3.Session().resource, service_name, instance_name, [], kwargs)
+
+    def aiostorage_client(self, **kwargs):
+        service_name = "aiostorage_client"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        scrts = self.secrets["storage"]
+        cnfg = self.config["storage"]
+        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
+        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
+        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
+        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
+        return self.get_instance(aioboto3.Session().client, service_name, instance_name, [], kwargs)
+
